from django.urls import path, re_path
from drf_yasg.views import get_schema_view
from drf_yasg import openapi
from rest_framework.routers import DefaultRouter
from rest_framework.permissions import AllowAny

from core import views

# Create a router and register our viewsets with it.
router = DefaultRouter()
router.register('student', views.StudentReviewerApiViewSet,
                basename='student')
router.register('examtype', views.ExamApiViewSet)
router.register('feedback', views.FeedbackApiView)
router.register('feedback-comment', views.FeedbackCommentApiView)
router.register('submission', views.SubmissionViewSet,
                basename='submission')
router.register('submissiontype', views.SubmissionTypeApiView)
router.register('tutor', views.TutorApiViewSet, basename='tutor')
router.register('subscription', views.SubscriptionApiViewSet,
                basename='subscription')
router.register('assignment', views.AssignmentApiViewSet)
router.register('statistics', views.StatisticsEndpoint, basename='statistics')
router.register('user', views.UserAccountViewSet, basename='user')

schema_view = get_schema_view(
    openapi.Info(
        title="Grady API",
        default_version='v1',
        description="Blub",
    ),
    # validators=['flex', 'ssv'],
    public=True,
    permission_classes=(AllowAny,),
)

# regular views that are not viewsets
regular_views_urlpatterns = [
    path('student-page/',
         views.StudentSelfApiView.as_view(),
         name='student-page'),
    path('student-submissions/',
         views.StudentSelfSubmissionsApiView.as_view(),
         name='student-submissions'),
    path('user-role/', views.get_user_role, name='user-role'),
    path('jwt-time-delta/',
         views.get_jwt_expiration_delta,
         name='jwt-time-delta'),
    path('instance/export/', views.InstanceExport.as_view(), name="instance-export"),
    path('export/json/', views.StudentJSONExport.as_view(), name='export-json'),
    re_path(r'swagger(?P<format>\.json|\.yaml)$',
            schema_view.without_ui(cache_timeout=0), name='schema-json'),
    re_path(r'swagger/$', schema_view.with_ui('swagger', cache_timeout=0),
            name='schema-swagger-ui'),
    re_path(r'redoc/$', schema_view.with_ui('redoc', cache_timeout=0),
            name='schema-redoc'),
]

urlpatterns = [
    *router.urls,
    *regular_views_urlpatterns,
]
