import '../../utils/setup'
import Vuex from 'vuex'
import { mount, createLocalVue } from '@vue/test-utils'
import DataExport from '@/components/export/DataExport.vue'
import sinon from 'sinon'
import chai from 'chai'
import VueRouter from 'vue-router'
import testUtils from '@/../tests/utils/testUtils'

import * as api from '@/api'
chai.should()

let localVue = createLocalVue()
localVue.use(Vuex)
localVue.use(VueRouter)
let router = new VueRouter()

// @ts-ignore
global.requestAnimationFrame = cb => cb()

describe('DataExport Component Unit Tests', () => {
  let store: any = null
  let consoleTemp = {
    warn: console.warn,
    error: console.error
  }

  before(function () {
    console.warn = function () {}
    console.error = function () {}
  })

  after(function () {
    console.warn = consoleTemp.warn
    console.error = consoleTemp.error
  })

  beforeEach(() => {
    store = new Vuex.Store({
      state: {}
    })
  })

  afterEach(() => {
    sinon.restore()
  })

  it('should download CSV when selected', () => {
    let wrapper = mount(DataExport, { localVue: localVue, store })
    wrapper.vm.$data.exportType = 'CSV'
    let spy = sinon.spy()
    // @ts-ignore
    sinon.replace(wrapper.vm, 'jsonToCSV', spy)
    // @ts-ignore
    sinon.replace(wrapper.vm, 'createDownloadPopup', sinon.spy())
    // @ts-ignore
    wrapper.vm.optionalConvertAndCreatePopup(testUtils.studentExports)
    spy.called.should.equal(true)
  })
  it('should download JSON when selected', () => {
    let wrapper = mount(DataExport, { localVue: localVue, store })
    wrapper.vm.$data.exportType = 'JSON'
    let spy = sinon.spy()
    // @ts-ignore
    sinon.replace(wrapper.vm, 'jsonToCSV', spy)
    // @ts-ignore
    sinon.replace(wrapper.vm, 'createDownloadPopup', sinon.spy())
    // @ts-ignore
    wrapper.vm.optionalConvertAndCreatePopup(testUtils.studentExports)
    spy.called.should.equal(false)
  })
  it('should download obfuscated data when no mapping was selected', async () => {
    let wrapper = mount(DataExport, { localVue: localVue, store })
    let stub = sinon.stub().returns(Promise.resolve({ data: testUtils.studentExports }))
    let spy = sinon.spy()
    // @ts-ignore replace ax.post because of fetch in getExportFile
    sinon.replace(api.default, 'post', stub)
    // @ts-ignore
    sinon.replace(wrapper.vm, 'optionalConvertAndCreatePopup', spy)
    // @ts-ignore
    await wrapper.vm.getExportFile('data')
    spy.calledWithExactly(testUtils.studentExports).should.equal(true)
  })
  it('should download deobfuscated data when mapping was selected', async () => {
    let wrapper = mount(DataExport, { localVue: localVue, store })
    let stub = sinon.stub().returns(Promise.resolve({ data: testUtils.studentExports }))
    let spy = sinon.spy()
    // @ts-ignore
    wrapper.vm.mapFile = testUtils.fakeFile
    // @ts-ignore replace ax.post because of fetch in getExportFile
    sinon.replace(api.default, 'post', stub)
    // @ts-ignore
    sinon.replace(wrapper.vm, 'getMappedExportFile', spy)
    // @ts-ignore
    await wrapper.vm.getExportFile('data')
    spy.calledWithExactly(testUtils.studentExports).should.equal(true)
  })
  it('should correctly apply the student-map file', () => {
    let wrapper = mount(DataExport, { localVue: localVue, store })
    let mapStub = sinon.stub().returns(testUtils.studentMap)
    let spy = sinon.spy()
    // @ts-ignore
    sinon.replaceGetter(wrapper.vm, 'studentMap', mapStub)
    // @ts-ignore
    sinon.replaceGetter(wrapper.vm, 'mapFileLoaded', sinon.stub().returns(true))
    // @ts-ignore
    sinon.replace(wrapper.vm, 'optionalConvertAndCreatePopup', spy)
    // @ts-ignore
    wrapper.vm.getMappedExportFile(testUtils.studentExportsObfuscated)
    spy.firstCall.args[0][0].Matrikel.should.equal(testUtils.studentMap['username'].matrikelNo)
  })
})
