import datetime
import logging
from django.test import LiveServerTestCase
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec

from core.models import UserAccount
from functional_tests.util import (login, create_browser)
from util import factory_boys as fact

log = logging.getLogger(__name__)


class TestAutoLogout(LiveServerTestCase):
    browser: webdriver.Firefox = None
    username = None
    password = None
    role = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.browser = create_browser()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        cls.browser.quit()

    def setUp(self):
        self.username = 'reviewer'
        self.password = 'p'
        self.role = UserAccount.TUTOR
        fact.UserAccountFactory(
            username=self.username,
            password=self.password,
            role=self.role
        )

    def _login(self):
        login(self.browser, self.live_server_url, self.username, self.password)

    def test_auto_logout_can_continue(self):
        with self.settings(JWT_AUTH={
            'JWT_EXPIRATION_DELTA': datetime.timedelta(seconds=15),
            'JWT_ALLOW_REFRESH': True,
        }
        ):
            self._login()
            initial_token = self.browser.execute_script(
                'return sessionStorage["token"]'
            )
            logout_dialog = self.browser.find_element_by_id('logout-dialog')
            WebDriverWait(self.browser, 15).until(
                ec.visibility_of_element_located((By.ID, 'logout-dialog')))
            logout_dialog.find_element_by_id('continue-btn').click()
            WebDriverWait(self.browser, 10).until(
                ec.invisibility_of_element_located((By.ID, 'logout-dialog')))
            self.assertNotIn('login', self.browser.current_url)
            new_token = self.browser.execute_script(
                'return sessionStorage["token"]'
            )
            self.assertNotEqual(initial_token, new_token)
