import os
from itertools import islice
from typing import Sequence

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.ui import WebDriverWait


def create_browser() -> webdriver.Firefox:
    options = Options()
    options.headless = bool(os.environ.get('HEADLESS_TESTS', False))
    browser = webdriver.Firefox(options=options)
    browser.implicitly_wait(10)
    browser.set_window_size(1920, 1080)
    return browser


def login(browser, live_server_url, username, password='p'):
    browser.get(live_server_url)
    username_input = browser.find_element_by_xpath('//input[@aria-label="Username"]')
    username_input.send_keys(username)
    password_input = browser.find_element_by_xpath('//input[@aria-label="Password"]')
    password_input.send_keys(password)
    browser.find_element_by_xpath('//button[@type="submit"]').send_keys(Keys.ENTER)
    WebDriverWait(browser, 10).until(ec.url_contains('/home'))


def reset_browser_after_test(browser: webdriver.Firefox, live_server_url):
    while len(browser.window_handles) > 1:
        browser.close()
    browser.switch_to.window(browser.window_handles[0])
    browser.execute_script("window.sessionStorage.clear()")
    browser.get(live_server_url)


def nth(iterable, n, default=None):
    "Returns the nth item or a default value"
    return next(islice(iterable, n, None), default)


def extract_hrefs_hashes(web_elements: Sequence[WebElement]):
    return [nth(el.get_attribute('href').split('#'), 1, '')
            for el in web_elements if el.get_attribute('href')]


# A function that takes the element corresponding to the tasks
# component and return a function that can be used as a condition for
# WebDriverWait
def subscriptions_loaded_cond(tasks_el):
    def loaded(*args):
        sub_links = tasks_el.find_elements_by_tag_name('a')
        return any((link != '/home' for link in extract_hrefs_hashes(sub_links)))
    return loaded
